import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.List;

/**
 * Space. Something for rockets to fly in.
 * 
 * @author Michael Kolling
 * @version 1.0
 */
public class Space extends World
{
    public Rocket rocket;
    public Counter scoreCounter;
    public Counter levelCounter;
//     public Counter reloadProtonWaveCounter;
    public int lives = 3;
    private int startAsteroids = 1;
    
    private static final int FIRST_X = 30;
    private static final int LIVES_SPACE = 30;
    
    private GreenfootSound backgroundMusic = new GreenfootSound("Masteroids.mp3");
    public MuteButton muteButton = new MuteButton();

    public Space() 
    {
        super(600, 600, 1);
        GreenfootImage background = getBackground();
        background.setColor(Color.BLACK);
        background.fill();
        backgroundMusic.playLoop();
        
        rocket = new Rocket();
        addObject(rocket, getWidth()/2 + 100, getHeight()/2);
        
        addAsteroids(startAsteroids);
        createLives(lives);
        
        levelCounter = new Counter("Level: ", 1);
        addObject(levelCounter, 60, getHeight()-40);
        scoreCounter = new Counter("Score: ");
        addObject(scoreCounter, 60, getHeight()-20);
//         reloadProtonWaveCounter = new Counter("Reload Proton Wave: ");
//         addObject(reloadProtonWaveCounter, 125, 380);

        Explosion.initializeImages();
        ProtonWave.initializeImages();
        
        createStars(300);
        addObject(new Instructions(), getWidth()/2, getHeight()/2);
        addObject(muteButton, getWidth()-30, getHeight()-30);
    }
    
    /**
     * Add a given number of asteroids to our world. Asteroids are only added into
     * the left half of the world.
     */
    private void addAsteroids(int count) 
    {
        for(int i = 0; i < count; i++) 
        {
            int x = Greenfoot.getRandomNumber(getWidth()/2);
            int y = Greenfoot.getRandomNumber(getHeight()/2);
            addObject(new Asteroid(), x, y);
        }
    }
    
    /**
     * Create a given number of lives in our world.
     */
    private void createLives(int count) 
    {
        for(int i = 0; i < count; i++) 
        {
            addObject(new Lives(FIRST_X, LIVES_SPACE), FIRST_X+i*LIVES_SPACE, 30);
        }
    }
    
    /**
     * Create some stars by the given int.
     */
    public void createStars(int number)
    {
       GreenfootImage background = getBackground();
       for(int i = 0; i < number; i++) {
          int x = Greenfoot.getRandomNumber(getWidth());
          int y = Greenfoot.getRandomNumber(getHeight());
          int rgb = Greenfoot.getRandomNumber(255);
          background.setColor( new Color(rgb, rgb, rgb) );
          background.fillOval(x, y, 2, 2);
       }
    }
    
    public void act()
    {
//        reloadProtonWaveCounter.setValue(rocket.timeToReloadProtonWave());
       createBonus(500);
       checkAsteroids();
    }
    
    /**
     * Create randomly bonuses in the world (only from level 3)
     */
    public void createBonus(int chance)
    {
       if( Greenfoot.getRandomNumber(chance) == 1  && getLevel() >= 3) {
          addObject(new Bonus(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
       }
    }
    
    /**
     * Checks the number of the asteroids in the world - if
     * there aren't asteroids at all create new ones.
     */
    public void checkAsteroids()
    {
       List<Asteroid> asteroids = getObjects(Asteroid.class);
       if(asteroids.size() == 0) {
          if(!muteButton.mute) Greenfoot.playSound("hooray.wav");
          Greenfoot.delay(30);
          startAsteroids++;
          levelCounter.add(1);
          addAsteroids(startAsteroids);
          rocket.setLocation(getWidth()/2 + 100, getHeight()/2);
          rocket.setShield(false);
          Greenfoot.delay(30);
       }
    }
    
    /**
     * Add score to the score counter.
     */
    public void countScore(int toAdd)
    {
       scoreCounter.add(toAdd);
    }
    /**
     * Return the number of the level.
     */
    public int getLevel()
    {
       return levelCounter.getValue();
    }
    
    /**
     * Return how many lives do we have.
     */
    public int getLives()
    {
       return lives;
    }
    /**
     * Add or remove lives from the rocket by a given int.
     */
    public void addLives(int toAdd)
    {
       for(int i = 0; i < toAdd; i++) 
       {
          addObject(new Lives(FIRST_X, LIVES_SPACE), FIRST_X+(lives+i)*LIVES_SPACE, 30);
       }
       lives += toAdd;
    }
    
    /**
     * Return the background music of the game.
     */
    public GreenfootSound getBackgroundMusic()
    {
       return backgroundMusic;
    }
    
    /**
     * This method is called when the game is over to display the final score.
     */
    public void gameOver() 
    {
        addObject(new ScoreBoard( getLevel(), scoreCounter.getValue() ), getWidth()/2, getHeight()/2);
    }
}